@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 
 type getarg > /dev/null 2>&1 || . /lib/dracut-lib.sh
 
@@ -32,9 +32,37 @@ overlay_size=$(getarg rd.live.overlay.size=)
 
 getargbool 0 rd.live.overlay.thin && thin_snapshot="yes"
 getargbool 0 rd.live.overlay.overlayfs && overlayfs="yes"
+join=$(getarg rd.live.join -d join)
 
+# Take a path to a disk label and return the parent disk if it is a partition
+# Otherwise returns the original path
+get_check_dev() {
+    local _udevinfo
+    dev_path="$(udevadm info -q path --name "$1")"
+    _udevinfo="$(udevadm info -q property --path "${dev_path}")"
+    strstr "$_udevinfo" "DEVTYPE=partition" || {
+        echo "$1"
+        return
+    }
+    parent="${dev_path%/*}"
+    _udevinfo="$(udevadm info -q property --path "${parent}")"
+    strstr "$_udevinfo" "DEVTYPE=disk" || {
+        echo "$1"
+        return
+    }
+    strstr "$_udevinfo" "ID_FS_TYPE=iso9660" || {
+        echo "$1"
+        return
+    }
+
+    # Return the name of the parent disk device
+    echo "$_udevinfo" | grep "DEVNAME=" | sed 's/DEVNAME=//'
+}
+
+# Find the right device to run check on
+check_dev=$(get_check_dev "$livedev")
 # CD/DVD media check
-[ -b "$livedev" ] && fs=$(blkid -s TYPE -o value "$livedev")
+[ -b "$check_dev" ] && fs=$(blkid -s TYPE -o value "$check_dev")
 if [ "$fs" = "iso9660" -o "$fs" = "udf" ]; then
     check="yes"
 fi
@@ -42,10 +70,10 @@ getarg rd.live.check -d check || check=""
 if [ -n "$check" ]; then
     type plymouth > /dev/null 2>&1 && plymouth --hide-splash
     if [ -n "$DRACUT_SYSTEMD" ]; then
-        p=$(dev_unit_name "$livedev")
+        p=$(dev_unit_name "$check_dev")
         systemctl start checkisomd5@"${p}".service
     else
-        checkisomd5 --verbose "$livedev"
+        checkisomd5 --verbose "$check_dev"
     fi
     if [ $? -eq 1 ]; then
         die "CD check failed!"
@@ -62,7 +90,7 @@ det_img_fs() {
     blkid -s TYPE -u noraid -o value "$1"
 }
 
-modprobe squashfs
+load_fstype squashfs
 CMDLINE=$(getcmdline)
 for arg in $CMDLINE; do
     case $arg in
@@ -88,20 +116,12 @@ else
         # no mount needed - we've already got the LiveOS image in $livedev
         SQUASHED=$livedev
     elif [ "$livedev_fstype" != "ntfs" ]; then
-        if ! mount -n -t "$fstype" -o "${liverw:-ro}" "$livedev" /run/initramfs/live; then
+        if ! mount -n -t "$livedev_fstype" -o "${liverw:-ro}" "$livedev" /run/initramfs/live; then
             die "Failed to mount block device of live image"
             exit 1
         fi
     else
-        # Symlinking /usr/bin/ntfs-3g as /sbin/mount.ntfs seems to boot
-        # at the first glance, but ends with lots and lots of squashfs
-        # errors, because systemd attempts to kill the ntfs-3g process?!
-        if [ -x "/usr/bin/ntfs-3g" ]; then
-            (exec -a @ntfs-3g ntfs-3g -o "${liverw:-ro}" "$livedev" /run/initramfs/live) | vwarn
-        else
-            die "Failed to mount block device of live image: Missing NTFS support"
-            exit 1
-        fi
+        [ -x "/sbin/mount-ntfs-3g" ] && /sbin/mount-ntfs-3g -o "${liverw:-ro}" "$livedev" /run/initramfs/live
     fi
 fi
 
@@ -129,7 +149,14 @@ do_live_overlay() {
     # need to know where to look for the overlay
     if [ -z "$setup" -a -n "$devspec" -a -n "$pathspec" -a -n "$overlay" ]; then
         mkdir -m 0755 -p /run/initramfs/overlayfs
-        mount -n -t auto "$devspec" /run/initramfs/overlayfs || :
+        if ismounted "$devspec"; then
+            devmnt=$(findmnt -e -v -n -o 'TARGET' --source "$devspec")
+            # We need $devspec writable for overlay storage
+            mount -o remount,rw "$devspec"
+            mount --bind "$devmnt" /run/initramfs/overlayfs
+        else
+            mount -n -t auto "$devspec" /run/initramfs/overlayfs || :
+        fi
         if [ -f /run/initramfs/overlayfs$pathspec -a -w /run/initramfs/overlayfs$pathspec ]; then
             OVERLAY_LOOPDEV=$(losetup -f --show ${readonly_overlay:+-r} /run/initramfs/overlayfs$pathspec)
             over=$OVERLAY_LOOPDEV
@@ -170,7 +197,7 @@ do_live_overlay() {
         fi
     fi
     if [ -n "$overlayfs" ]; then
-        if ! modprobe overlay; then
+        if ! load_fstype overlay; then
             if [ "$overlayfs" = required ]; then
                 die "OverlayFS is required but not available."
                 exit 1
@@ -188,25 +215,37 @@ do_live_overlay() {
         elif [ -n "$devspec" -a -n "$pathspec" ]; then
             [ -z "$m" ] \
                 && m='   Unable to find a persistent overlay; using a temporary one.'
-            m="$m"$'\n      All root filesystem changes will be lost on shutdown.'
-            m="$m"$'\n         Press [Enter] to continue.'
+            m="$m"'
+      All root filesystem changes will be lost on shutdown.
+         Press [Enter] to continue.'
             printf "\n\n\n\n%s\n\n\n" "${m}" > /dev/kmsg
             if [ -n "$DRACUT_SYSTEMD" ]; then
                 if type plymouth > /dev/null 2>&1 && plymouth --ping; then
                     if getargbool 0 rhgb || getargbool 0 splash; then
-                        m='>>>'$'\n''>>>'$'\n''>>>'$'\n\n\n'"$m"
-                        m="${m%n.*}"$'n.\n\n\n''<<<'$'\n''<<<'$'\n''<<<'
+                        m='>>>
+>>>
+>>>
+
+
+'"$m"
+                        m="${m%n.*}"'n.
+
+
+<<<
+<<<
+<<<'
                         plymouth display-message --text="${m}"
                     else
                         plymouth ask-question --prompt="${m}" --command=true
                     fi
                 else
-                    m=">>>${m//.[[:space:]]/.}  <<<"
+                    m=">>>$(printf '%s' "$m" | tr -d '\n')  <<<"
                     systemd-ask-password --timeout=0 "${m}"
                 fi
             else
                 type plymouth > /dev/null 2>&1 && plymouth --ping && plymouth --quit
-                read -s -r -p $'\n\n'"${m}" -n 1 _
+                printf '\n\n%s' "$m"
+                read -r _
             fi
         fi
         if [ -n "$overlayfs" ]; then
@@ -274,6 +313,9 @@ do_live_overlay() {
 if [ -e /run/initramfs/live/${live_dir}/${squash_image} ]; then
     SQUASHED="/run/initramfs/live/${live_dir}/${squash_image}"
 fi
+if [ -e /run/initramfs/live/${live_dir}/${join} ]; then
+    SQUASHED_JOIN="/run/initramfs/live/${live_dir}/${join}"
+fi
 if [ -e "$SQUASHED" ]; then
     if [ -n "$live_ram" ]; then
         echo 'Copying live image to RAM...' > /dev/kmsg
@@ -288,6 +330,13 @@ if [ -e "$SQUASHED" ]; then
     mkdir -m 0755 -p /run/initramfs/squashfs
     mount -n -t squashfs -o ro "$SQUASHED_LOOPDEV" /run/initramfs/squashfs
 
+    if [ -n "${join}" ]; then
+        SQUASHED_JOIN_LOOPDEV=$(losetup -f)
+        losetup -r "$SQUASHED_JOIN_LOOPDEV" $SQUASHED_JOIN
+        mkdir -m 0755 -p /run/initramfs/joinfs
+        mount -n -t squashfs -o ro "$SQUASHED_LOOPDEV" /run/initramfs/joinfs
+    fi
+
     if [ -d /run/initramfs/squashfs/LiveOS ]; then
         if [ -f /run/initramfs/squashfs/LiveOS/rootfs.img ]; then
             FSIMG="/run/initramfs/squashfs/LiveOS/rootfs.img"
@@ -296,6 +345,7 @@ if [ -e "$SQUASHED" ]; then
         fi
     elif [ -d /run/initramfs/squashfs/proc ]; then
         FSIMG=$SQUASHED
+        FSIMG_JOIN=$SQUASHED_JOIN
         if [ -z "$overlayfs" ] && [ -n "$DRACUT_SYSTEMD" ]; then
             reloadsysrootmountunit=":>/xor_overlayfs;"
         fi
@@ -311,6 +361,9 @@ else
     elif [ -e /run/initramfs/live/${live_dir}/ext3fs.img ]; then
         FSIMG="/run/initramfs/live/${live_dir}/ext3fs.img"
     fi
+    if [ -e /run/initramfs/live/${live_dir}/${join} ]; then
+        FSIMG_JOIN="/run/initramfs/live/${live_dir}/${join}"
+    fi
     if [ -n "$live_ram" ]; then
         echo 'Copying live image to RAM...' > /dev/kmsg
         echo ' (this may take a minute or so)' > /dev/kmsg
@@ -369,26 +422,13 @@ if [ -n "$overlayfs" ]; then
     if [ -n "$FSIMG" ]; then
         mkdir -m 0755 -p /run/rootfsbase
         mount -r $FSIMG /run/rootfsbase
+        if [ -n "${join}" ]; then
+            mkdir -m 0755 /run/rootfsjoin
+            mount -r $FSIMG_JOIN /run/rootfsjoin
+        fi
     else
         ln -sf /run/initramfs/live /run/rootfsbase
     fi
-    mkdir -m 0755 -p /run/overlayfs
-    mkdir -m 0755 -p /run/ovlwork
-    if [ -n "$reset_overlay" ] && [ -h /run/overlayfs ]; then
-        ovlfs=$(readlink /run/overlayfs)
-        info "Resetting the OverlayFS overlay directory."
-        rm -r -- "${ovlfs:?}"/* "${ovlfs:?}"/.* > /dev/null 2>&1
-    fi
-    if [ -n "$readonly_overlay" ] && [ -h /run/overlayfs-r ]; then
-        ovlfs=lowerdir=/run/overlayfs-r:/run/rootfsbase
-    else
-        ovlfs=lowerdir=/run/rootfsbase
-    fi
-    if [ -z "$DRACUT_SYSTEMD" ]; then
-        printf 'mount -t overlay LiveOS_rootfs -o%s,%s %s\n' "$ROOTFLAGS" \
-            "$ovlfs",upperdir=/run/overlayfs,workdir=/run/ovlwork \
-            "$NEWROOT" > "$hookdir"/mount/01-$$-live.sh
-    fi
 else
     if [ -z "$DRACUT_SYSTEMD" ]; then
         [ -n "$ROOTFLAGS" ] && ROOTFLAGS="-o $ROOTFLAGS"
